<?php
/*!
 * C-Rent Web
 *
 * Copyright 2011, CX9 Systems GmbH
 * Authors: Christoph Weitkamp - c.weitkamp@cx9.de
 *
 * http://www.cx9.de
 */
// Class definition
class JasperPrint extends DBBase
{

    /**
     * see if the java extension was loaded.
     */
    public function checkJavaExtension()
    {
        // check extension
        if (!extension_loaded("java")) {
            $sapi_type = php_sapi_name();
            $port = (isset($_SERVER["SERVER_PORT"]) && (($_SERVER["SERVER_PORT"]) > 1024)) ? $_SERVER["SERVER_PORT"] : "8080";
            if ($sapi_type == "cgi" || $sapi_type == "cgi-fcgi" || $sapi_type == "cli") {
                if (!(PHP_SHLIB_SUFFIX == "so" && @dl("java.so")) && !(PHP_SHLIB_SUFFIX == "dll" && @dl("php_java.dll")) && !(@include_once "java/Java.inc") && !(include_once "http://127.0.0.1:$port/java/Java.inc")) {
                    return "java extension not installed.";
                }
            } else {
                if (!(@include_once "java/Java.inc")) {
                    include_once "http://127.0.0.1:$port/java/Java.inc";
                }
            }
        }
        if (!function_exists("java_get_server_name")) {
            return "The loaded java extension is not the PHP/Java Bridge";
        }
        // check java
        try {
            new Java("java.util.HashMap");
        } catch (Exception $e) {
            return $e->getMessage();
        }

        return true;
    }

    /**
     * convert a php value to a java one...
     * @param string $value
     * @param string $className
     * @returns boolean success
     */
    public function convertValue($value, $className)
    {
        // if we are a string, just use the normal conversion
        // methods from the java extension...
        try {
            if ($className == "java.lang.String") {
                $temp = new Java("java.lang.String", $value);
                return $temp;
            } elseif ($className == "java.lang.Boolean" ||
                $className == "java.lang.Integer" ||
                $className == "java.lang.Long" ||
                $className == "java.lang.Short" ||
                $className == "java.lang.Double" ||
                $className == "java.math.BigDecimal") {
                $temp = new Java($className, $value);
                return $temp;
            } elseif ($className == "java.sql.Timestamp" ||
                $className == "java.sql.Time") {
                $temp = new Java($className);
                $javaObject = $temp->valueOf($value);
                return $javaObject;
            }
        } catch (Exception $err) {
            echo ("unable to convert value, " . $value . " could not be converted to " . $className);
            return false;
        }

        echo ("unable to convert value, class name " . $className . " not recognised");
        return false;
    }

    /**
     * Erzeugt und konfiguriert die Verbindung zur Datenbank.
     *
     * @return java.sql.Connection      Ein java.sql.Connetion-Objekt mit Informationen ueber die
     *                                  Verbindung zur Datenbank
     */
    private function getDBConnection()
    {
        include MODUL_PATH . 'config/config_db.inc.php';
        // Der java.sql.DriverManager stellt im Jasper-Ausdruck die Verbindung zur Datenbank her
        $class = new JavaClass("java.lang.Class");
        $class->forName("net.sourceforge.jtds.jdbc.Driver");
        $driverManager = new JavaClass("java.sql.DriverManager");

        // Erzeuge das Passwort fuer die Datzenbankverbindung, wenn nicht gegeben
        if (empty($password)) {
            // Passwort wird aus dem benutzernamen generiert
            $password = DBConnection::get_password($username);
        }

        if (!empty($instance)) {
            $hostWithoutInstance = str_replace('\\' . $instance, '', $host);
            $connection = $driverManager->getConnection('jdbc:jtds:sqlserver://' . $hostWithoutInstance . (empty($port) ? '' : ':' . $port) . '/' . $dbname . ';instance=' . $instance, $username, $password);
        } else {
            $connection = $driverManager->getConnection('jdbc:jtds:sqlserver://' . $host . (empty($port) ? '' : ':' . $port) . '/' . $dbname, $username, $password);
        }

        unset($driverManager);
        unset($class);

        return $connection;
    }

    /**
     * Compiliert ein Jrxml-Template in das zugehoerige Jasper-Template.
     *
     * @param string $templatePath        Pfad des Jrxml-Templates
     * @param string $outputPath          Pfad fuer die Ausgabe der Jasper-Datei
     */
    public function compile($templatePath, $outputPath)
    {
        // Der net.sf.jasperreports.engine.JasperCompileManager konvertiert die Jasper-Templates im
        // jrxml-Format in das zugehoerige Jasper-Template
        $compileManager = new JavaClass("net.sf.jasperreports.engine.JasperCompileManager");
        $compileManager->compileReportToFile($templatePath, $outputPath);

        unset($compileManager);
    }

    /**
     * Erstellt eine PDF-Datei ueber die Jasper-Bibliothek.
     *
     * @param string $reportPath        Pfad des Jasper-Templates
     * @param string $outputPath        Pfad fuer die Ausgabe der PDF-Datei
     * @param java.util.HashMap $params Eine java.util.HashMap mit Parametern, die an den Jasper-Ausdruck
     *                                  uebergeben werden sollen
     */
    public function buildPDF($reportPath, $outputPath, $params)
    {
        // Erzeuge die Verbindung zur Datenbank
        $connection = $this->getDBConnection();

        // Der net.sf.jasperreports.engine.JasperFillManager weist dem Jasper-Template die Parameter und
        // die Datenbankverbindung zu
        $fillManager = new JavaClass("net.sf.jasperreports.engine.JasperFillManager");
        $jasperPrint = $fillManager->fillReport($reportPath, $params, $connection);

        // Der net.sf.jasperreports.engine.JasperExportManager erstellt die gewuenschte PDF-Datei
        $exportManager = new JavaClass("net.sf.jasperreports.engine.JasperExportManager");
        $exportManager->exportReportToPdfFile($jasperPrint, $outputPath);

        unset($exportManager);
        unset($fillManager);
        unset($connection);
    }

    /**
     * Erstellt eine HTML-Datei ueber die Jasper-Bibliothek.
     *
     * @param string $reportPath        Pfad des Jasper-Templates
     * @param string $outputPath        Pfad fuer die Ausgabe der HTML-Datei
     * @param java.util.HashMap $params Eine java.util.HashMap mit Parametern, die an den Jasper-Ausdruck
     *                                  uebergeben werden sollen
     */
    public function buildHTML($reportPath, $outputPath, $params)
    {
        // Erzeuge die Verbindung zur Datenbank
        $connection = $this->getDBConnection();

        // Der net.sf.jasperreports.engine.JasperFillManager weist dem Jasper-Template die Parameter und
        // die Datenbankverbindung zu
        $fillManager = new JavaClass("net.sf.jasperreports.engine.JasperFillManager");
        $jasperPrint = $fillManager->fillReport($reportPath, $params, $connection);

        // Der net.sf.jasperreports.engine.JasperExportManager erstellt die gewuenschte HTML-Datei und die
        // zugehoerigen Bilder etc.
        $exportManager = new JavaClass("net.sf.jasperreports.engine.JasperExportManager");
        $exportManager->exportReportToHtmlFile($jasperPrint, $outputPath);

        unset($exportManager);
        unset($fillManager);
        unset($connection);
    }
}
