<?php
class PayPalPlus
{
    public $mode;
    public $payPalMode;
    public $apiUser;
    public $apiPassword;
    public $apiSignature;
    public $logoUrl;
    public $response = null;

    public function __construct()
    {
        $params = new Params();
        $this->mode = $params->getvALUE('PAYPAL_MODE');
        if ($this->mode == 1) {
            $this->apiUser = $params->getvALUE('PAYPAL_API_USERNAME');
            $this->apiPassword = $params->getvALUE('PAYPAL_API_PASSWORD');
            $this->apiSignature = $params->getvALUE('PAYPAL_API_SIGNATURE');
            $this->payPalMode = '';
        } else {
            $this->apiUser = $params->getvALUE('PAYPAL_TEST_API_USERNAME');
            $this->apiPassword = $params->getvALUE('PAYPAL_TEST_API_PASSWORD');
            $this->apiSignature = $params->getvALUE('PAYPAL_TEST_API_SIGNATURE');
            $this->payPalMode = '.sandbox';
        }
    } // __construct

    public function getAuthToken()
    {
        $ch = curl_init();
        curl_setopt($ch, CURLOPT_URL, 'https://api' . $this->payPalMode . '.paypal.com/v1/oauth2/token');
        curl_setopt($ch, CURLOPT_HEADER, false);
        curl_setopt($ch, CURLOPT_SSL_VERIFYPEER, false);
        curl_setopt($ch, CURLOPT_POST, true);
        curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
        curl_setopt($ch, CURLOPT_USERPWD, $this->apiSignature . ":" . $this->apiPassword);
        curl_setopt($ch, CURLOPT_POSTFIELDS, "grant_type=client_credentials");
        $result = curl_exec($ch);
        $json = json_decode($result);
        $accessToken = $json->access_token;
        curl_close($ch);

        if (isset($json->name) or isset($json->error)) {
            return false;
        }

        return $accessToken;
    }

    public function getExperienceProfileId($accessToken, $name)
    {
        $ch = curl_init();
        $headers = array('Content-Type:application/json', 'Authorization:Bearer ' . $accessToken);
        curl_setopt($ch, CURLOPT_HTTPHEADER, $headers);
        curl_setopt($ch, CURLOPT_URL, 'https://api' . $this->payPalMode . '.paypal.com/v1/payment-experience/web-profiles');
        curl_setopt($ch, CURLOPT_HEADER, false);
        curl_setopt($ch, CURLOPT_SSL_VERIFYPEER, false);
        curl_setopt($ch, CURLOPT_CUSTOMREQUEST, 'POST');
        curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
        curl_setopt($ch, CURLOPT_POSTFIELDS, '{
         "name": "' . $name . time() . '",
         "presentation": {
         "brand_name": "' . $name . '",
         "logo_image": "' . $this->logoUrl . '",
         "locale_code": "DE"
         },
         "input_fields": {
         "allow_note": false,
         "no_shipping": 1,
         "address_override": 1
         }
        }');
        $result = curl_exec($ch);
        $json = json_decode($result);

        if (isset($json->name) or isset($json->error)) {
            return false;
        }

        $experienceProfileId = $json->id;
        curl_close($ch);

        return $experienceProfileId;
    }

    public function createPayment($accessToken, $returnUrl, $cancelUrl, $amount, $currency = 'EUR', $description = '', $experienceProfileId = '')
    {
        $ch = curl_init();
        $headers = array('Content-Type:application/json', 'Authorization:Bearer ' . $accessToken);
        curl_setopt($ch, CURLOPT_HTTPHEADER, $headers);
        curl_setopt($ch, CURLOPT_URL, 'https://api' . $this->payPalMode . '.paypal.com/v1/payments/payment');
        curl_setopt($ch, CURLOPT_HEADER, false);
        curl_setopt($ch, CURLOPT_SSL_VERIFYPEER, false);
        curl_setopt($ch, CURLOPT_CUSTOMREQUEST, 'POST');
        curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
        $jsons = '{
          "intent":"sale",' . $experienceProfileId . '
          "redirect_urls":{
            "return_url":"' . $returnUrl . '",
            "cancel_url":"' . $cancelUrl . '"
          },
          "payer":{
            "payment_method":"paypal"
          },
          "transactions":[
            {
              "amount":{
                "total":"' . $amount . '",
                "currency":"' . $currency . '"
              }, "description": "' . urlencode($description) . '"
            }
          ]
        }';
        curl_setopt($ch, CURLOPT_POSTFIELDS, $jsons);
        $result = curl_exec($ch);
        $json = json_decode($result);

        if (isset($json->name) or isset($json->error)) {
            return false;
        }

        $linkPay = $json->links[1]->href;

        return $linkPay;
    }

    public function executePayment($accessToken, $paymentId, $payerId)
    {
        $ch = curl_init();
        $headers = array('Content-Type:application/json', 'Authorization:Bearer ' . $accessToken);
        curl_setopt($ch, CURLOPT_HTTPHEADER, $headers);
        curl_setopt($ch, CURLOPT_URL, 'https://api' . $this->payPalMode . '.paypal.com/v1/payments/payment/' . $paymentId . '/execute');
        curl_setopt($ch, CURLOPT_HEADER, false);
        curl_setopt($ch, CURLOPT_SSL_VERIFYPEER, false);
        curl_setopt($ch, CURLOPT_CUSTOMREQUEST, 'POST');
        curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
        $jsons = '{ "payer_id" : "' . $payerId . '" }';
        curl_setopt($ch, CURLOPT_POSTFIELDS, $jsons);
        $result = curl_exec($ch);
        $json = json_decode($result);
        $payed = $json->transactions[0]->related_resources[0]->sale->state;
        $saleId = $json->transactions[0]->related_resources[0]->sale->id;

        if (isset($json->name) or isset($json->error)) {
            return false;
        }

        return ($payed == 'completed') ? $saleId : false;
    }

    public function getErrorStatus($accessToken, $paymentId)
    {
        $ch = curl_init();
        $headers = array('Content-Type:application/json', 'Authorization:Bearer ' . $accessToken);
        curl_setopt($ch, CURLOPT_HTTPHEADER, $headers);
        curl_setopt($ch, CURLOPT_URL, 'https://api' . $this->payPalMode . '.paypal.com/v1/payments/payment/' . $paymentId);
        curl_setopt($ch, CURLOPT_HEADER, false);
        curl_setopt($ch, CURLOPT_SSL_VERIFYPEER, false);
        curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
        $result = curl_exec($ch);
        $json = json_decode($result);

        return $json;
    }
}
