<?php

class Pdfprint
{
    private $jasper;
    private $lang;
    private $path;
    private $logofile;
    private $outputPath;
    private $reportPath;
    private $jasper_template;
    private $filename;
    private $subdir;
    private $pdf_delete = 1;
    private $params = array();

    public function __construct()
    {
        $this->jasper = new JasperPrint();
        $this->jasper->checkJavaExtension();

        $this->lang = Session::getLang();

        $print_path = dirname(MODUL_PATH) . '/reservierung/print/';
        if (!is_dir($print_path)) {
            $print_path = dirname(MODUL_PATH) . '/crentweb/print/';
        }

        if (!is_dir($print_path)) {
            $print_path = dirname(MODUL_PATH) . '/print/';
        }

        if (!is_dir($print_path)) {
            $print_path = APP_PATH . 'print/';
        }

        if (!is_dir($print_path)) {
            goError('Print order not found!');
        }

        $this->path = rtrim($print_path, '/') . '/';
    } // __construct()

    public function __destruct()
    {
        if (file_exists($this->outputPath) && $this->pdf_delete == 1) {
            unlink($this->outputPath);
        }
    } // __destruct()

    public function setPrintDir($path)
    {
        $this->path = rtrim($path, '/') . '/';
    } // setPrintPath()

    public function setPrintSubdir($path)
    {
        $this->subdir = trim($path, '/') . '/';
    } // setPrintSubdir()

    public function setFilename($filename)
    {
        $this->filename = $filename;
    } // setPrintSubdir()

    public function setJasperTemplate($jasperTemplate)
    {
        $this->jasper_template = $jasperTemplate;
    } // setJasperTemplate()

    public function addParam($param, $value)
    {
        $this->params[$param] = $value;
    }

    private function createPDF()
    {
        $this->path = $this->path . $this->subdir . $this->lang . '/';

        if (!file_exists($this->path)) {
            goError('Pdfprint::Jasper template path not found!');
        }

        $this->logofile = $this->path . 'LOGO.jpg';

        if (empty($this->filename)) {
            $this->filename = date('YmdHis') . '.pdf';
        }

        $this->reportPath = $this->path . $this->jasper_template;
        if (!file_exists($this->reportPath)) {
            goError('Pdfprint::Jasper template not found!');
        }

        $this->outputPath = $this->path . $this->filename;

        $params = new Java('java.util.HashMap');
        switch ($this->jasper_template) {
            case 'rechnung.jasper':
            case 'gutschein_rechnung.jasper':
                if (!empty($this->params['reno'])) {
                    $params->put('RENUMMER', $this->jasper->convertValue($this->params['reno'], 'java.lang.String'));
                } else {
                    die('Pdfprint::Wrong invoice number!');
                }
                break;
            case 'gutschein.jasper':
                if (!empty($this->params['gsno'])) {
                    $params->put('VOUCHER_CODE', $this->jasper->convertValue($this->params['gsno'], 'java.lang.String'));
                    $params->put('VOUCHER_ANREDE', $this->jasper->convertValue($this->params['anrede'], 'java.lang.String'));
                    $params->put('VOUCHER_COMMENT', $this->jasper->convertValue($this->params['comment'], 'java.lang.String'));
                } else {
                    goError('Pdfprint::Wrong voucher number!');
                }
                break;
            case 'reservierung.jasper':
                if (!empty($this->params['resno'])) {
                    $params->put('VERTRAGSNUMMER', $this->jasper->convertValue($this->params['resno'], 'java.lang.Integer'));
                } else {
                    goError('Pdfprint::Wrong reservation number!');
                }
                break;
        }

        $params->put('SUBREPORT_DIR', $this->jasper->convertValue($this->path, 'java.lang.String'));

        if (file_exists($this->logofile)) {
            $params->put('LOGO', $this->jasper->convertValue($this->logofile, 'java.lang.String'));
        }

        $this->jasper->buildPDF($this->reportPath, $this->outputPath, $params);
    } // createPDF()

    public function printPDF()
    {
        $this->createPDF();

        if (file_exists($this->outputPath)) {
            header('Content-type: application/pdf; charset=UTF-8');
            header('Content-Length: ' . filesize($this->outputPath));
            header('Content-Disposition: attachment; filename="' . $this->filename . '"');
            header('Content-Transfer-Encoding: binary');
            header('Cache-Control: must-revalidate');
            header('Pragma: no-cache');
            header('Expires: 0');
            readfile($this->outputPath);
            exit;
        }
    } // printPDF()

    public function getPDFtoString()
    {
        $this->createPDF();
        return file_get_contents($this->outputPath);
    } // getPDFtoString()
} // Class Pdfprint
