<?php
class DBCarGroup extends DBBase
{
    public function getAllGroups($station_dep, $station_arr, $start_time, $end_time, $customer_number = 0)
    {
        $dbTranslate = new DBTranslate();
        $translate = $dbTranslate->loadTranslations('GRUPPE', $this->registry->lang);
        $car_groups = array();
        $params = array(
            'station' => $station_dep->code,
            'von' => $start_time,
            'bis' => $end_time,
        );
        if ($customer_number > 0) {
            $params['KDNR'] = $customer_number;
        }
        $result = $this->getDatabase()->proc_exec('INET_GET_KFZGROUPS', $params);
        if ($result !== false) {
            $all_groups = $this->getDatabase()->fetch_all();
            foreach ($all_groups as $value) {
                $group = new \stdClass;
                $group->gruppe = $value['gruppe'];
                $group->title = $value['title'];
                $group->text_web = $value['text_web'];
                $group->sortierung = $value['sortierung'];
                $group->kfzart = $value['kfzart'];
                $group->kfzkategorie = $value['kfzkategorie'];
                $group->text_web2 = $value['text_web2'];
                $group->sizeOfInet_bild = $value['sizeOfInet_bild'];
                $group->sizeOfKFZ_INET_AUSST_PIC = $value['sizeOfKFZ_INET_AUSST_PIC'];
                if ($group->title == '') {
                    $tmp = $this->getGroupByCode($group->gruppe);
                    $group->title = $tmp->Beschreibung;
                }
                $group->title_lang = !empty($translate['TITLE'][md5($group->gruppe)]) ? $translate['TITLE'][md5($group->gruppe)] : $group->title;
                $group->kfzkategorie_lang = '';
                foreach (explode(';', $group->kfzkategorie) as $cat) {
                    if (!empty($translate['KATEGORIE'][md5($cat)])) {
                        $group->kfzkategorie_lang .= $translate['KATEGORIE'][md5($cat)] . ';';
                    } else {
                        $group->kfzkategorie_lang .= $cat . ';';
                    }
                }
                $group->kfzkategorie_lang = trim($group->kfzkategorie_lang, ';');
                $car_groups[] = $group;
            }
        }
        return $car_groups;
    } // getAllGroups()

    public function getGroupByCode($group_code)
    {
        $dbObj = null;
        $sql = 'SELECT gruppe, text_web, sortierung, kfzart, kfzkategorie, Beschreibung , Beschreibung_WEB , KFZ_KATEGORIE_TEXT FROM AUTOS_KFZ_GRUPPEN WHERE gruppe = :group AND Inetgruppe = 1';
        $this->getDatabase()->prepare($sql);
        $this->getDatabase()->bindValue(':group', $group_code);

        $result = $this->getDatabase()->execute();

        if ($result !== false) {
            $dbObj = $this->getDatabase()->fetch_obj();
        }
        return $dbObj;
    } // getGroupByCode()

    public function getSizeOfBild($group_code, $img_typ)
    {
        switch ($img_typ) {
            case 'ext':
                $feld = 'KFZ_INET_AUSST_PIC';
                break;
            case 'group':
                $feld = 'inet_bild';
                break;
            default:
                return false;
        }
        $dbObj = null;
        $sql = "SELECT  ISNULL(DATALENGTH(:feld), 0) AS :img_typ
					FROM AUTOS_KFZ_GRUPPEN WHERE gruppe = :group";
        $this->getDatabase()->prepare($sql);
        $this->getDatabase()->bindValue(':group', $group_code);
        $this->getDatabase()->bindValue(':feld', $feld);
        $this->getDatabase()->bindValue(':img_typ', $img_typ);

        $result = $this->getDatabase()->execute();

        if ($result !== false) {
            $dbObj = $this->getDatabase()->fetch_obj();
        }
        return $dbObj;
    } //getSizeOfBild
    public function getSizeGroupImage($group_code, $image_pk, $db_field_alias = 'image_size')
    {
        $dbObj = null;
        $sql = "SELECT TOP 1 ISNULL(DATALENGTH(Bild), 0) AS :db_field_alias FROM dbo.AUTOS_VK_BILDER WHERE Gruppe = :group AND PK = :image_pk";

        $this->getDatabase()->prepare($sql);
        $this->getDatabase()->bindValue(':db_field_alias', $db_field_alias);
        $this->getDatabase()->bindValue(':group', $group_code);
        $this->getDatabase()->bindValue(':image_pk', $image_pk);

        $result = $this->getDatabase()->execute();

        if ($result !== false) {
            $dbObj = $this->getDatabase()->fetch_array()[0];
        }
        return $dbObj;
    } //getSizeGroupImages

    public function getImageGroup($group_code, $image_pk, $db_field_alias = 'image', $getIdent = false)
    {
        $dbObj = null;
        $sql = ($getIdent)
        ? 'SELECT PK , DATEDIFF(second,\'1970-01-01 00:00:00\',TIMESTAMP) AS ident FROM dbo.AUTOS_VK_BILDER WHERE Gruppe = :group AND PK = :image_pk'
        : 'SELECT TOP 1 ISNULL(Bild, \'\') AS :db_field_alias, ISNULL(Sortierung,0) AS Sortierung FROM dbo.AUTOS_VK_BILDER WHERE Gruppe = :group AND PK = :image_pk';

        $this->getDatabase()->prepare($sql);
        $this->getDatabase()->bindValue(':group', $group_code);
        $this->getDatabase()->bindValue(':image_pk', $image_pk);
        if (!$getIdent) {
            $this->getDatabase()->bindValue(':db_field_alias', $db_field_alias);
        }

        $result = $this->getDatabase()->execute();
        if ($result !== false) {
            if ($getIdent) {
                $tmp = $this->getDatabase()->fetch_assoc();
                $dbObj = $tmp['ident'] . '_' . $tmp['PK'];
            } else {
                $dbObj = $this->getDatabase()->fetch_array()[0];
            }
        }
        return $dbObj;
    } //getImageGroup

    public function getImageGroupURIs($group_code, $db_field_alias = 'image_name', $group_sufix = '/group_image/big/')
    {
        $dbObj = null;
        $sql = "SELECT Gruppe+:group_sufix+CAST(PK AS VARCHAR(16)) AS :db_field_alias FROM AUTOS_VK_BILDER WHERE Gruppe = :group ORDER BY Sortierung ASC";
        $this->getDatabase()->prepare($sql);
        $this->getDatabase()->bindValue(':group', $group_code);
        $this->getDatabase()->bindValue(':group_sufix', $group_sufix);
        $this->getDatabase()->bindValue(':db_field_alias', $db_field_alias);

        $result = $this->getDatabase()->execute();

        if ($result !== false) {
            $dbObj = $this->getDatabase()->fetch_all();
        }
        return $dbObj;
    } //getImageGroup

    public function getImage($group_code, $img_typ = 'group')
    {
        switch ($img_typ) {
            case 'ext':
                $feld = 'KFZ_INET_AUSST_PIC';
                break;

            case 'group':
                $feld = 'inet_bild';
                break;

            default:
                return false;
        }

        $sql = 'SELECT ' . $getValue . ' FROM AUTOS_KFZ_GRUPPEN WHERE gruppe = :group';
        $this->getDatabase()->prepare($sql);
        $this->getDatabase()->bindValue(':group', $group_code);
        $result = $this->getDatabase()->execute();

        if ($result !== false) {
            $this->getDatabase()->bindColumn(1, $image, PDO::PARAM_LOB);
            $row = $this->getDatabase()->fetch_bound();
            return $image == null ?false : $image;
        } else {
            return false;
        }
    } // getImage()

    public function getBelegungsPlan($group_code, $datetime_dep, $datetime_arr, $station = '%', $alternative_pruefung = 0)
    {
        $belegungsplan = array();
        $query = "
						EXEC dbo.GET_Fahrzeuggruppe_at_least_one_free
						@von = :von,						-- datetime
					    @bis = :bis,						-- datetime
					    @gruppe = :gruppe,					-- varchar(255)
						@vorlaufzeit_unix =	0,			-- Bigint Zeit in Sekunden (3600 entspricht einer Stunde)
						@nachlaufzeit_unix = 0,			-- Bigint Zeit in Sekunden
						@karenz = 0,					-- Bigint Zeit in Sekunden
						@station_übergabe = :station,	-- varchar (255)
						@alternative_pruefung = :alternative_pruefung";

        $this->getDatabase()->prepare($query);
        $this->getDatabase()->bindValue(':von', $datetime_dep);
        $this->getDatabase()->bindValue(':bis', $datetime_arr);
        $this->getDatabase()->bindValue(':gruppe', $group_code);
        $this->getDatabase()->bindValue(':station', $station);
        $this->getDatabase()->bindValue(':alternative_pruefung', $alternative_pruefung);
        $result = $this->getDatabase()->execute();

        if ($result !== false) {
            if ($alternative_pruefung == 0) {
                while ($row = $this->getDatabase()->fetch_assoc()) {
                    //Ergebniss ist bei der Alternativen Prüfung das TOP 1 Fahrzeug und keine Zeit
                    $belegungsplan[] = array('start' => substr($row['VON'], 0, 16), 'end' => substr($row['BIS'], 0, 16), 'activ_cars' => substr($row['aktiveautosfuergruppe'], 0, 3));
                }
                return $belegungsplan;
            } else {
                $auto = array();
                while ($row2 = $this->getDatabase()->fetch_assoc()) {
                    $auto[] = array('mvanummer' => $row2['mvanummer']);
                }
                return $auto;
            }
        }
    } // getBelegungsPlan()

    public function getGroupCharacteristics($group_code)
    {
        $merkmale = array();
        $sql = 'SELECT A.PK AS aUid , A.KFZGRUPPE , A.MERKMAL AS customMerkmal , A.WERT , A.inet , B.PK AS bUid , B.MERKMAL AS dbMerkmal , B.cssClass FROM AUTOS_KFZ_GRUPPEN_MERKMALE AS A LEFT JOIN AUTOS_GRUPPEN_MERKMALE AS B ON A.PKMERKMAL = B.PK WHERE A.inet = 1 AND KFZGRUPPE = :group ORDER BY bUid';
        $this->getDatabase()->prepare($sql);
        $this->getDatabase()->bindValue(':group', $group_code);
        $result = $this->getDatabase()->execute();
        if ($result !== false) {
            while ($row = $this->getDatabase()->fetch_assoc()) {
                $merkmale[] = new CarGroupCharacteristics($row);
            }
        }

        return $merkmale;
    } //getGroupCharacteristics()

    public function getGroupsFromRate($rate_pk)
    {
        $groups = array();
        $sql = "SELECT gruppe, Beschreibung_WEB as title, text_web,
			   		          sortierung, kfzart, kfzkategorie, KFZ_KATEGORIE_TEXT AS text_web2,
			   		          DATALENGTH(inet_bild) AS sizeOfInet_bild, DATALENGTH(KFZ_INET_AUSST_PIC) AS sizeOfKFZ_INET_AUSST_PIC
			   		   FROM AUTOS_KFZ_GRUPPEN
			   		   LEFT JOIN RECH_RATEN ON RECH_RATEN.Grp = AUTOS_KFZ_GRUPPEN.Gruppe
			   		   WHERE RECH_RATEN.Rate = (SELECT Rate FROM RECH_RATEN WHERE PK = " . (int) $rate_pk . ")
			   		   ORDER BY AUTOS_KFZ_GRUPPEN.Sortierung";
        $result = $this->getDatabase()->query($sql);
        if ($result !== false) {
            $db_groups = $this->getDatabase()->fetch_all();

            foreach ($db_groups as $dbCarGroup) {
                $car_group = new \stdClass;
                $car_group->gruppe = $dbCarGroup['gruppe'];
                $car_group->title = $dbCarGroup['title'];
                $car_group->text_web = $dbCarGroup['text_web'];
                $car_group->text_web2 = $dbCarGroup['text_web2'];
                $car_group->sortierung = $dbCarGroup['sortierung'];
                $car_group->kfzart = $dbCarGroup['kfzart'];
                $car_group->kfzkategorie = $dbCarGroup['kfzkategorie'];
                $car_group->sizeOfInet_bild = $dbCarGroup['sizeOfInet_bild'];
                $car_group->sizeOfKFZ_INET_AUSST_PIC = $dbCarGroup['sizeOfKFZ_INET_AUSST_PIC'];

                if ($car_group->title == '') {
                    $tmp = $this->getGroupByCode($group->gruppe);
                    $car_group->title = $tmp->Beschreibung;
                }

                $car_group->title_lang = !empty($translate['TITLE'][md5($car_group->gruppe)]) ? $translate['TITLE'][md5($car_group->gruppe)] : $car_group->title;
                $car_group->kfzkategorie_lang = '';

                foreach (explode(';', $car_group->kfzkategorie) as $cat) {
                    if (!empty($translate['KATEGORIE'][md5($cat)])) {
                        $car_group->kfzkategorie_lang .= $translate['KATEGORIE'][md5($cat)] . ';';
                    } else {
                        $car_group->kfzkategorie_lang .= $cat . ';';
                    }
                }
                $car_group->kfzkategorie_lang = trim($car_group->kfzkategorie_lang, ';');

                $groups[] = $car_group;
            }
        }

        return $groups;
    } // getGroupsFromRate()

    public function isGroupAvailable($startdate, $enddate, $starttime, $endtime, $station_code, $group)
    {
        $start_datetime = date('d.m.Y H:i', strtotime($startdate . ' ' . $starttime));
        $end_datetime = date('d.m.Y H:i', strtotime($enddate . ' ' . $endtime));
        $starttime = date('Hi', strtotime($start_datetime));
        $endtime = date('Hi', strtotime($end_datetime));
        $startdate = date('d.m.Y', strtotime($start_datetime));
        $enddate = date('d.m.Y', strtotime($end_datetime));
        $result = false;
        $response = $this->getBelegungsPlan($group, $start_datetime, $end_datetime, $station_code, 1);
        if ($response[0]['mvanummer'] == null) {
            $result = false;
        } else {
            $result = $response[0]['mvanummer'];
        }
        return $result;
    } //isGroupAvailable()

    public function getReservierungen($group_code)
    {
        $reservierungen = array();
        $sql = "SELECT MIETVERTRAEGE.MVANUMMER, VERTRAGSNUMMER,
                dbo.Dattodt(ABDAT, ABZEIT) AS ABDATETIME,
                dbo.Dattodt(VEREINBRUECKGABEDAT, VEREINRUECKZEIT) AS ANDATETIME
                FROM MIETVERTRAEGE LEFT JOIN dbo.aktive
                ON dbo.MIETVERTRAEGE.MVANUMMER = dbo.aktive.MVANr
                WHERE aktive.Gruppe = :group AND dbo.Dattodt(VEREINBRUECKGABEDAT, VEREINRUECKZEIT) >= CURRENT_TIMESTAMP
                ORDER BY MIETVERTRAEGE.MVANUMMER ASC, ABDATETIME ASC";

        $this->getDatabase()->prepare($sql);
        $this->getDatabase()->bindValue(':group', $group_code);
        $this->getDatabase()->execute();
        $result = $this->getDatabase()->execute();
        if ($result !== false) {
            while ($row = $this->getDatabase()->fetch_assoc()) {
                $reservierungen[] = $row;
            }
        }
        return $reservierungen;
    } // getReservierungen

    public function getAutos($group_code)
    {
        $autos = array();
        $sql = 'select * from aktive where GRUPPE = :group';
        $this->getDatabase()->prepare($sql);
        $this->getDatabase()->bindValue(':group', $group_code);
        $this->getDatabase()->execute();
        $result = $this->getDatabase()->execute();
        if ($result !== false) {
            while ($row = $this->getDatabase()->fetch_assoc()) {
                $autos[] = $row;
            }
        }
        return $autos;
    } //getAutos()

    public function getAuto($mvanummer)
    {
        $sql = 'SELECT * FROM  autos WHERE MVANUMMER = :mvanummer';
        $this->getDatabase()->prepare($sql);
        $this->getDatabase()->bindValue(':mvanummer', $mvanummer);
        $result = $this->getDatabase()->execute();
        if ($result !== false) {
            $auto = $this->getDatabase()->fetch_obj();
        }

        if ($auto === false) {
            $auto = new \stdClass;
            $auto->MVANUMMER = $mvanummer;
        }
        return $auto;
    } //getAuto()

    public function getPartkplatz($mvanummer)
    {

        $sql = 'SELECT Parkplatz as Parkplatz FROM  autos WHERE MVANUMMER = :mvanummer';
        $this->getDatabase()->prepare($sql);
        $this->getDatabase()->bindValue(':mvanummer', $mvanummer);
        $result = $this->getDatabase()->execute();

        if ($result !== false) {
            $parkplatz = $this->getDatabase()->fetch_assoc()['Parkplatz'];
        }
        return $parkplatz;
    } // getPartkplatz()
} // Class DBCarGroup
