<?php
class DBStation extends DBBase
{
    public function getAllStations($angpk = 0, $region = '')
    {
        $stations = array();
        $region_stations = array();
        if (!empty($region)) {
            $sql = 'SELECT Filiale FROM DISPO_REL_REGIONEN_STATIONEN WHERE region = :region';
            $this->getDatabase()->prepare($sql);
            $this->getDatabase()->bindValue(':region', $region);
            $result = $this->getDatabase()->execute();
            if ($result !== false) {
                while ($row = $this->getDatabase()->fetch_assoc()) {
                    $region_stations[] = $row['Filiale'];
                }
            }
        }

        $result = $this->getDatabase()->proc_exec('INET_GET_ALL_STATIONS_WITH_INFO', array('ANGPK' => $angpk));

        if ($result !== false) {
            while ($station = $this->getDatabase()->fetch_obj()) {
                if (empty($region) || in_array($station->station, $region_stations)) {
                    $stations[] = $station;
                }
            }
        }

        foreach ($stations as &$station) {
            $station->land = $this->getLand($station->station);
        }
        unset($station);

        return $stations;
    }

    public function getStationByCode($code)
    {
        $station = false;
        $result = $this->db->proc_exec('INET_GET_STATIONS_INFO', array('station' => $code));

        if ($result !== false) {
            $station = $this->db->fetch_obj();
            if (!empty($station)) {
                $sql = "SELECT INET_MIN_VORLAUF,url, isOneWay FROM dbo.stammt WHERE station = :code";
                $this->db->prepare($sql);
                $this->db->bindValue(':code', $code);
                $result = $this->db->execute();
                if ($result !== false) {
                    $row = $this->db->fetch_assoc();
                }
                $station->url = is_null($row['url']) ? '' : $row['url'];
                $station->min_vorlauf = is_null($row['INET_MIN_VORLAUF']) ? 0 : (int) $row['INET_MIN_VORLAUF'];
                $station->land = $this->getLand($code);
                $station->station = $code;
                $station->onlyCI = 0;
                $station->isOneWay = $row['isOneWay'];
            }
        }

        return $station;
    }

    public function getStationSchedule($code)
    {
        $dbTranslate = new DBTranslate;
        $translate = $dbTranslate->loadTranslations('Wochentage', $this->registry->lang);
        if (DEBUG || empty($_SESSION['CACHE']['dbschedule'][$code])) {
            $schedule = array();
            $result = $this->getDatabase()->proc_exec('INET_GET_FILIALE_SCHEDULE', array('station' => $code));

            if ($result !== false) {
                while ($db_schedule = $this->getDatabase()->fetch_obj()) {
                    $tmp = new StationSchedule();
                    $tmp->day = $db_schedule->wd;
                    $tmp->btime = $db_schedule->bt;
                    $tmp->etime = $db_schedule->et;
                    $tmp->mbtime = !empty($db_schedule->bm) ? $db_schedule->bm : '0000';
                    $tmp->metime = !empty($db_schedule->em) ? $db_schedule->em : '0000';

                    if ($tmp->btime == $tmp->etime) {
                        $tmp->stationClosed = true;
                    }

                    switch ($tmp->day) {
                        case 'Mo':
                            $tmp->day_lang = $translate['Montag'];
                            break;
                        case 'Di':
                            $tmp->day_lang = $translate['Dienstag'];
                            break;
                        case 'Mi':
                            $tmp->day_lang = $translate['Mittwoch'];
                            break;
                        case 'Do':
                            $tmp->day_lang = $translate['Donnerstag'];
                            break;
                        case 'Fr':
                            $tmp->day_lang = $translate['Freitag'];
                            break;
                        case 'Sa':
                            $tmp->day_lang = $translate['Samstag'];
                            break;
                        case 'So':
                            $tmp->day_lang = $translate['Sonntag'];
                            break;
                    }

                    $schedule[] = $tmp;
                }
            }
            $_SESSION['CACHE']['dbschedule'][$code] = serialize($schedule);
        }

        return unserialize($_SESSION['CACHE']['dbschedule'][$code]);
    }

    public function getLand($station_code)
    {
        $land = '';

        $this->getDatabase()->prepare('SELECT * FROM stammt WHERE station = :station');
        $this->getDatabase()->bindValue(':station', $station_code);
        $result = $this->getDatabase()->execute();

        if ($result !== false) {
            $row = $this->db->fetch_assoc();
            if (!empty($row['land'])) {
                $sql = "SELECT land FROM LAENDERKENNZEICHEN
                            WHERE Kennzeichen = :land";
                $this->db->prepare($sql);
                $this->db->bindValue(':land', $row['land']);
                $result = $this->db->execute();
                if ($result !== false) {
                    $row = $this->getDatabase()->fetch_assoc();
                    if (!empty($row['land'])) {
                        $land = $row['land'];
                    }
                }
            }
        }

        return $land;
    } // getLand()

    public function is_open($station_code, $date)
    {
        $is_open = false;
        $time = date('Hi', strtotime($date));
        $date = date('Y-m-d', strtotime($date));
        $wt = date('N', strtotime($date)) - 1;
        $sh = $this->getStationSchedule($station_code);
        $is_open = $time >= $sh[$wt]->btime && $time <= $sh[$wt]->etime;
        if ($is_open && ($sh[$wt]->mbtime != $sh[$wt]->metime)) {
            $is_open = $time < $sh[$wt]->mbtime || $time > $sh[$wt]->metime;
        }
        return $is_open;
    } // is_open()

    public function isCheckSumm()
    {
        $ch_sum1 = 0;
        $ch_sum2 = 1;
        $result = $this->getDatabase()->query('SELECT CHECKSUM_AGG(BINARY_CHECKSUM(strasse, plz, ort)) as st_chsum FROM stammt ');
        if ($result !== false) {
            $row = $this->getDatabase()->fetch_assoc();
            $ch_sum1 = $row['st_chsum'];
        }

        $result = $this->getDatabase()->query("SELECT Text FROM next_update WHERE Updatename = 'STATIONS_POSITION'");
        if ($result !== false) {
            $row = $this->getDatabase()->fetch_assoc();
            if (empty($row)) {
                $this->getDatabase()->query("INSERT INTO dbo.next_update ( Updatename, LastUpdate, Text )
                                        VALUES  ( 'STATIONS_POSITION',  GETDATE(), '')");
            } else {
                $ch_sum2 = $row['Text'];
            }
        }

        return intval($ch_sum1) == intval($ch_sum2);
    }

    public function setCheckSum()
    {
        $this->getDatabase()->query("UPDATE next_update
                             SET LastUpdate = GETDATE(),
                             Text = (SELECT TOP 1 CHECKSUM_AGG(BINARY_CHECKSUM(strasse, plz, ort)) FROM stammt)
                             WHERE Updatename = 'STATIONS_POSITION'");
    } // setCheckSum()

    public function positionUpdate($station, $lat, $long)
    {
        $sql = 'UPDATE stammt SET INET_GOOGLE_MAPS_LINK = :position WHERE station = :station';
        $this->getDatabase()->prepare($sql);
        $this->getDatabase()->bindValue(':position', ($lat . ',' . $long));
        $this->getDatabase()->bindValue(':station', $station);
        $this->getDatabase()->execute();
    } //positionUpdate()

    public function getReturnStations($code)
    {
        $stations = array();
        $sql = "SELECT station FROM stammt WHERE station NOT IN (SELECT nach FROM DOM_FIL_ENTF WHERE von = :code AND OneWayErlaubt = 0)";
        $this->getDatabase()->prepare($sql);
        $this->getDatabase()->bindValue(':code', $code);
        $result = $this->getDatabase()->execute();
        if ($result !== false) {
            while ($row = $this->getDatabase()->fetch_assoc()) {
                $stations[] = $row['station'];
            }
        }
        return $stations;
    } // getReturnStations()

    public function getZentrale()
    {
        $station = null;
        $this->db->query('SELECT Zentrale FROM MAND_GLOBAL LEFT JOIN dbo.stammt ON MAND_GLOBAL.Zentrale = stammt.station WHERE Internetstation = 1');

        $row = $this->db->fetch_assoc();
        if (!empty($row['Zentrale'])) {
            $station_id = $row['Zentrale'];
        } else {
            $this->db->query('SELECT TOP 1 station FROM stammt WHERE Internetstation = 1');
            $row = $this->db->fetch_assoc();
            if (!empty($row['station'])) {
                $station_id = $row['station'];
            }
        }
        if (!empty($station_id)) {
            $station = $this->getStationByCode($station_id);
        }
        return $station;
    } // getZentrale()

    public function getStationscodeByVertragsnummer($vertragsnummer = 0)
    {
        if ($vertragsnummer == 0) {
            return null;
        }
        $code = null;
        $sql = "SELECT VERMIETSTATION FROM dbo.MIETVERTRAEGE where VERTRAGSNUMMER = :vertragsnummer";
        $this->getDatabase()->prepare($sql);
        $this->getDatabase()->bindValue(':vertragsnummer', $vertragsnummer);
        $result = $this->getDatabase()->execute();
        if ($result !== false) {
            $field = $this->getDatabase()->fetch_assoc();
            $code = $field['VERMIETSTATION'];
        }
        return $code;
    }

    public function getStationscodeByReservierungsnummer($reservierungsnummer = 0)
    {
        if ($reservierungsnummer == 0) {
            return null;
        }
        $station_code = null;
        $sql = "SELECT VERMIETSTATION FROM dbo.MIETVERTRAEGE WHERE RESERVIERUNGSNUMMER = :reservierungsnummer";
        $this->getDatabase()->prepare($sql);
        $this->getDatabase()->bindValue(':reservierungsnummer', $reservierungsnummer);
        $result = $this->getDatabase()->execute();
        if ($result !== false) {
            $row = $this->getDatabase()->fetch_assoc();
            $station_code = $row['VERMIETSTATION'];
        }
        return $station_code;
    }
}
