<?php

abstract class ControllerBase extends CX9Logging
{

    protected $registry;
    protected $template;
    protected $error = array();
    protected $session_error = false;
    protected $params;
    protected $translationsModules = array(
        'MESSAGE', 'FZG', 'RENTAL', 'Wochentage', 'Wochentage_kurz', 'Monate_kurz', 'Feiertage', 'ERROR', #step:index (alt)
        'Einheiten', 'KATEGORIE', 'GRUPPE', 'NAVI', 'CURRENCY', #step:cars (alt)
        'RATE', #step:prices (alt)
        'RESERVATE', 'WEBEMAIL', #step:summary (alt)
        #step:payprocess (alt)
        'CONFIRM', #step:confirm (alt)
        'ANMIETDATEN_BUTTON', 'BUCHUNGSFILTER', 'BUCHUNGSLISTE', 'CAR_GROUP', 'EMAIL', 'ERRORS', 'EXTRAS', 'FILTER',
        'GLOBAL', 'LOGIN_BAR', 'LOGIN', 'NEW_PASSWORD', 'RABATT', 'SIDEBAR', 'STATION_BLOCK', 'STATION_SELECT', 'STATION',
        'STEP_MENU', 'STEP1', 'STEP3', 'STEP5', 'STEP6', 'TARIF', 'TITLE', 'TOP_LINK', 'USERDATA',
    );

    public function __construct()
    {

        parent::__construct();

        $this->registry = Registry::getInstance();

        $this->template = $this->registry->template;

        $this->params = $this->registry->params;

        if (!empty($_SESSION['error'])) {
            $this->session_error = true;
            $this->error = $_SESSION['error'];
            $_SESSION['error'] = array();
            unset($_SESSION['error']);
        }
    }

    abstract public function index();

    protected function loadTranslations($l10nModules)
    {
        $translates = array();
        $dbTranslate = new DBTranslate();

        if (!is_array($l10nModules) || count($l10nModules) == 0) {
            return false;
        }

        foreach ($l10nModules as $curModule) {
            $translates[$curModule] = $dbTranslate->loadTranslations($curModule, $this->registry->lang);
        }

        return $translates;
    }

    protected function loadRentForm()
    {
        $template = $this->template;

        $translate = $this->loadTranslations(array('FZG', 'Einheiten', 'KATEGORIE', 'GRUPPE', 'RENTAL', 'NAVI', 'Wochentage',
            'Wochentage_kurz', 'Monate_kurz', 'Feiertage', 'CURRENCY', 'GLOBAL', 'LOGIN_BAR', 'LOGIN', 'STEP_MENU',
            'BUCHUNGSFILTER', 'TOP_LINK', 'SIDEBAR', 'FILTER', 'ANMIETDATEN_BUTTON', 'STATION_SELECT',
            'STATION', 'CAR_GROUP', 'TARIF', 'EXTRAS', 'RABATT'));

        $tpl_translate = $template->get_template_vars('TRANSLATE');
        foreach ($tpl_translate as $k => $v) {
            if (!key_exists($k, $translate)) {
                $translate[$k] = $v;
            }
        }

        $startdatetime = (float) Session::getStartDateTime() * 1000;
        $enddatetime = (float) Session::getEndDateTime() * 1000;

        $station_dep = Session::getStartStation();
        $station_arr = Session::getEndStation();

        $stations_dep = Station::getStationListFromCache('departure');
        $stations_arr = Station::getStationListFromCache('arrival');

        $js_array = array('System.js', 'Libs.js', 'CX9Contract.js');
        $tpl_js_array = $template->get_template_vars('JS_FILES');
        $js_array = array_unique(array_merge($js_array, $tpl_js_array));
        sort($js_array);

        $template->assign('JS_FILES', $js_array);

        $template->assign('STATION_DEP', $station_dep);
        $template->assign('STATION_ARR', $station_arr);

        $template->assign('STATIONS_DEP', $stations_dep);
        $template->assign('STATIONS_ARR', $stations_arr);

        $template->assign('STARTDATETIME', $startdatetime);
        $template->assign('ENDDATETIME', $enddatetime);

        $template->assign('LANG', $this->registry->lang);

        $template->assign('TRANSLATE', $translate);
    }

    public function getProcedureParametersFromArray($inputArray, $procedureType = 'NORMAL', $inputArrayPrefix = '', $hex_fields = array())
    {
        $stringResult = "";
        if (is_array($inputArray)) {
            if ($inputArrayPrefix != "") {
                $inputArray = $this->removePrefixOfArrayKeys($inputArray, $inputArrayPrefix);
            }

            foreach ($inputArray as $key => &$value) {
                if ($procedureType == 'PDO') {
                    if (in_array($key, $hex_fields)) {
                        //for documents
                        $stringResult .= "@" . $key . " = " . (($value === '' || $value === 'NULL') ? "NULL" : "0x" . $value) . ", ";
                    } else {
                        $stringResult .= "@" . $key . " = :" . $key . ", ";
                    }
                } else {
                    if (in_array($key, $hex_fields)) {
                        //for documents
                        $stringResult .= "@" . $key . " = " . (($value === '' || $value === 'NULL') ? "NULL" : "0x" . $value) . ", ";
                    } else {
                        $stringResult .= "@" . $key . " = '" . $value . "', ";
                    }
                }
            }
        }
        return rtrim($stringResult, ' ,');
    }

    public function catNameHash($category_name)
    {
        return substr(md5($category_name), 0, 6);
    }

    public function getPDOArrayVariablesFromArray($inputArray, $inputArrayPrefix, $hex_fields = array())
    {
        $arrayResult = array();
        if (is_array($inputArray)) {
            if ($inputArrayPrefix != "") {
                $inputArray = $this->removePrefixOfArrayKeys($inputArray, $inputArrayPrefix);
            }

            foreach ($inputArray as $key => &$value) {
                // documents will not be checked by PDO -> Possible bugs
                if (!in_array($key, $hex_fields)) {
                    $arrayResult[':' . $key] = $value;
                }
            }
        }
        return $arrayResult;
    }
    public function removePrefixOfArrayKeys($inputArray, $prefix = '#[a-z]+[_]#')
    {
        $newArray = array(); // new array to store key value pairs in
        foreach ($inputArray as $key => $value) {
            $newKeyName = preg_replace($prefix, '', $key); // removes as default the "word_" prefix from the initial key name
            $newArray[$newKeyName] = $value;
        }
        return $newArray;
    }
    public function process_request($user_input_data, $procedure_name, $proc_prefix, $unset_array_values = array(), $files = array(''), $callback = 'null', $exitOnCallback = false)
    {
        //extract($user_input_data, EXTR_PREFIX_SAME, "same_prefix_");
        //get procedure sufixes
        if (defined('PROCEDURE_SUFIXES')) {
            $procedure_information = unserialize(PROCEDURE_SUFIXES); // get information of executing class and function : Who is executing this method?
            $executed_by_class = debug_backtrace()[1]['class'];
            $executed_by_function = debug_backtrace()[1]['function'];
            $procedure_sufix = isset($procedure_information[$executed_by_class][$executed_by_function][$procedure_name]) ? $procedure_information[$executed_by_class][$executed_by_function][$procedure_name] : '';
            $procedure_name .= $procedure_sufix;
        }
        $data_result = array();
        $unset_array_values = array_merge($unset_array_values, array('hash', 'timestamp', 'take_logged_in_session'));
        $procedureData = $user_input_data; //copy original post array to
        if (!isset($timestamp)) {
            $timestamp = "";
        }

        if (strpos($procedure_name, 'dbo.') === false) {
            $procedure_name = 'dbo.' . $procedure_name;
        }

        // print_r($user_input_data);
        if (defined('ORM_CONTROLLER_NO_LOGIN')) {
            $procedure_information = array(); // get information of executing class and function : Who is executing this method?
            $controller_class = debug_backtrace()[1]['class'];
            $controller_method = debug_backtrace()[1]['function']; //Slider -> GetData //array('Slider'=>array('getGroups','getGroups','getCars','getData','getOtherCars'));
            $login = false;
            if (isset($procedure_information[$controller_class])) {
                $login = in_array($controller_method, $procedure_information[$controller_class]) ? true : false;
            }

            if ($login || ORM_CONTROLLER_NO_LOGIN) {
                $loginStatus['STATUS'] = 'LOGIN';
                $loginStatus['MESSAGE'] = '';
                $loginStatus['print_reachability'] = '0';
            }
        } else {
            // $loginStatus = $this->process_login($user_input_data)['loginStatus'];
            $loginStatus['STATUS'] = 'LOGIN';
            $loginStatus['MESSAGE'] = '';
            $loginStatus['print_reachability'] = '0';
        }

        if ($loginStatus['STATUS'] == 'LOGIN') {
            // if(!defined(CONTROLLER_PREFIXES)) define ('CONTROLLER_PREFIXES',serialize(array()));
            $prefixes = array();
            $data_result = array();
            $data_result['loginStatus'] = $loginStatus;
            $proc_prefix = (isset($proc_prefix) && $proc_prefix !== '') ? $proc_prefix : $prefixes['DEFAULT_PREFIX'];
            $procedureData['field_prefix'] = $proc_prefix;
            foreach ($unset_array_values as $key => $value) {
//delete unnecesary data from procedureData
                unset($procedureData[$value]);
            }
            $clientSearch = "EXEC " . $procedure_name . " " . $this->getProcedureParametersFromArray($procedureData, 'NORMAL', '[' . $proc_prefix . ']', $files);
            $pdo_query_string = "EXEC " . $procedure_name . " " . $this->getProcedureParametersFromArray($procedureData, "PDO", '[' . $proc_prefix . ']', $files);
            $pdo_array = $this->getPDOArrayVariablesFromArray($procedureData, '[' . $proc_prefix . ']', $files);
            $DBQueries = array("query" => $clientSearch, "pdo_query" => $pdo_query_string, "pdo_parameters" => $pdo_array);
            $db_data = new getProcData;
            //print_r($DBQueries);
            $data_result['result'] = $db_data->run($DBQueries, 1);
            //print_r($data_result);
            $time = date('Y-m-d H:i:s');
            //$this->setRequestInfo(get_class($this),$procedureData, $personSearchQueries, $data_result);
            if (defined('ORM_DEBUG_ALL_PROC_ACTIONS')) {
                $procedure_name_suffix = ($procedure_name . '[@ACTION=>') . (array_key_exists('aktion', $procedureData) ? $procedureData['aktion'] . ']' : (array_key_exists('action', $procedureData) ? $procedureData['action'] . ']' : 'DEFAULT]'));
            } else {
                $procedure_name_suffix = $procedure_name;
            }

            // $this->setRequestData(get_class($this),array($procedure_name_suffix =>array('CLIENT_INPUT_'.$time=>$procedureData, 'SERVER_SQL_'.$time=>$DBQueries, 'DB_RESULTS_'.$time=>$data_result)));
            $this->setRequestData(get_class($this), array($procedure_name_suffix => array('CLIENT_INPUT_' . $time => $procedureData, 'SERVER_SQL_' . $time => $DBQueries, 'DB_RESULTS' => $data_result)));
            //if(ENCODE_UTF8_TRANSLATIONS) $data_result = $format->encode_array_to_UTF8($data_result);
            if (is_callable($callback)) {
                $data_result = call_user_func($callback, $data_result, $procedureData);
            }

            if ($exitOnCallback) {
                return;
            }
        } else {
            $data_result = array('loginStatus' => $loginStatus);
        }

        $json = new JSONData($data_result);
        $json->print_json_headers();
        $json->print_json();
    }
    public function process_login($user_input_data)
    {
        //extract($user_input_data, EXTR_PREFIX_SAME, "same_prefix_");
        $data_result = array();
        $language = isset($user_input_data['language']) ? $user_input_data['language'] : $this->globalLanguage;
        $take_logged_in_session = isset($take_logged_in_session) ? $take_logged_in_session : 0;
        // require_once WORKING_DIR.'/libs/Autorization.php';
        // $cx9Auth = new Autorization();
        // $cx9Auth->getRSAKeys();
        if (!isset($timestamp)) {
            $timestamp = "";
        }

        $CRWLoginAction = isset($CRWLoginAction) ? $CRWLoginAction : 'LOGIN';
        $loginStatus = $cx9Auth->login($username, $hash, $timestamp, $take_logged_in_session, $CRWLoginAction, $language);
        //print_r($loginStatus);
        if ($loginStatus['STATUS'] == 'LOGIN') {
            return array('loginStatus' => $loginStatus); // print_r($loginStatus);
        } else {
            $this->view->json_headers();
            $data_result = array('loginStatus' => $loginStatus);
            $json = new JSONData($data_result);
            $json->print_json_headers();
            $json->print_json();
            die;
        }
    }
    public function destroy_session()
    {
        session_unset();
        session_destroy();
        echo "<div style='color:" . $this->random_color() . "'><b>Success at " . date('Y-m-d H:i:s.u') . '</b></div>';
    }
    protected function setRequestInfo($variable = 'var', $input = null, $pdo_array = null, $result = null)
    {
        if (isset($input) && !empty($input)) {
            $_SESSION[$variable]['input'] = $input;
        }

        
        if (isset($pdo_array) && !empty($pdo_array)) {
            $_SESSION[$variable]['queries'] = $pdo_array;
        }

        if (isset($result) && !empty($result)) {
            $_SESSION[$variable]['result'] = $result;
        }
    }
    protected function setRequestData($variable = 'var', $data)
    {
        if (is_array($data)) {
            foreach ($data as $key => $value) {
                if (isset($key) && !empty($key)) {
                    $_SESSION[$variable][$key] = $value;
                }
            }
        }
    }
    private function random_color_part()
    {
        return str_pad(dechex(mt_rand(0, 70)), 2, '0', STR_PAD_LEFT);
    }

    private function random_color()
    {
        return '#a320' . $this->random_color_part();
    }
    private function color($index)
    {
        $array = array('green', 'blue', '#0b163c');
        return $array[$index];
    }
    public function debug_model()
    {
        $session_variable = (isset($_GET['variable']) && $_GET['variable'] !== "") ? $_GET['variable'] : get_class($this);
        $color = (isset($_GET['color'])) ? filter_var($_GET['color'], FILTER_VALIDATE_BOOLEAN) : false;
        echo '<pre>';
        if (ORM_DEBUG && isset($_SESSION[$session_variable]) && $session_variable != 'RSA_KEYS') {
            $data = $this->XML2Array($_SESSION[$session_variable]);
            $mainKeys = array_keys($data);
            //border:1px solid gray;
            if ($color) {
                foreach ($mainKeys as $key => $value) {
                    $keys = array_keys($data[$value]);
                    echo '<div style=" font-size: large; border:1px solid gray; background-color: lavender; display: inline-block;"><b>';
                    echo 'PROCEDURE: ' . $value . '<br/>';
                    $i = 0;
                    foreach ($keys as $k => $v) {
                        echo $v . '<br/>';
                        echo '<div style=" font-size: large; color:' . $this->color($i) . '"><b>';
                        print_r($data[$value][$v]);
                        echo '</b></div>';
                        $i++;
                    }
                    echo '</b></div><br/>';
                }
            } else {
                print_r($data);
            }
        } else {
            echo 'no_data';
        }
    }

    private function XML2Array($input)
    {
        if (is_array($input)) {
            foreach ($input as $key => $value) {
                if (is_array($value)) {
                    $input[$key] = $this->XML2Array($value);
                } elseif (is_string($input[$key]) && $input[$key] != "") {
                    @$xml = simplexml_load_string($input[$key]);
                    if ($xml !== false) {
                        $input[$key] = json_decode(json_encode((array) $xml));
                    }
                }
            } //end of foreach
        } else {
            @$xml = simplexml_load_string($input);
            if ($xml !== false) {
                $input = json_decode(json_encode($xml), true);
            }
        }
        return $input;
    }

    /**
     *   die Standard round()-Funktion von PHP hat Probleme beim Runden von berechneten Werten, die auf .5 / .05 / .005 usw enden und eigentlich aufgerundet werden müssten.
     *   Problem ist, dass .5 sich binär nicht exakt darstellen lässt (ergibt annähernd .499999999), so dass dann eben ab- statt aufgerundet wird.
     **/
    protected function cx9round($term, $stellen)
    {
        switch ((int) $stellen) {
            case 0:
                return round((float) $term + 0.1, 0);
                break;
            case 2:
                return round((float) $term + 0.001, 2);
                break;
            default:
                return round((float) $term, (int) $stellen);
                break;
        }
    }

    public function checkLogin()
    {
        if (defined('FORCE_CUSTOMER_LOGIN') && FORCE_CUSTOMER_LOGIN == 1 && !isset($_SESSION['DATA']['login']['isLoggedIn']) && !isset($_SESSION['DATA']['customer'])) {
            header('location: ' . SITEURL . 'login');
            exit;
        }
    }
}
